/* 
 * PPRacer 
 * Copyright (C) 2004 Volker Stroebel <volker@planetpenguin.de>
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
 
#ifndef _PP_WIDGET_H
#define _PP_WIDGET_H

#include "alg/signal.h"
#include "alg/vec2d.h"

#include <SDL.h>

namespace pp {

class Widget {
	
protected:

    pp::Vec2d position;
    pp::Vec2d size;
	
	///is the box currently pushed down?
	bool clicked;
    ///is the mouse pointer currently in the box?
	bool focused;
    /// can the box be clicked?
	bool enabled;
    ///is the box being drawn by the UI manager?
	bool visible;
    /// is the box receiving mouse events?
	bool active;

	bool hasFocus;

	virtual void performClickAction();
		
public:
	Widget();
	virtual ~Widget();

	void setEnabled(bool enabled);
	void setVisible(bool visible);
	void setActive(bool active);
	void setPosition(pp::Vec2d pos){this->position = pos;};

	bool isInBox(const int x, const int y) const;

	void mouseDown(int button, int x, int y );
	void mouseUp(int button, int x, int y );
	void mouseMotion(int x, int y);
	virtual bool keyboardEvent(SDLKey key, SDLMod mod, bool release){return false;};
		
	void removeFocus();

	void setFocus(bool focus);

	void simulateMouseClick();

	virtual void draw()	= 0;

	inline double getWidth() const {return size.x;};
	inline double getHeight() const {return size.y;};

	//signals
	pp::Signal0 signalClicked;
};

} //namespace pp

#endif // _PP_WIDGET_H
